/*-----copyright-----
 *     amy-studio - make development easier 
 *     web        : http://www.amy-studio.com
 *     taobao     : http://amy-studio.taobao.com
 *     qq(e_mail) : amystudio@qq.com
 *-----file info-----
 *     name       : vga_drive.v
 *     author     : Yuphone Chang
 *     description: VGA display drvive
 *----- version -----
 *     created    : 7.20, 2010
 *     version    : 1.0
 *     description: original version
 *
 *     modified   : 
 *     version    : 
 *     description: 
 *-------------------
 */
module vga_drive(  
  input        i_clk,                 // global clock 
  input        i_rst_n,               // asynchronous reset 
  //
  output [9:0] o_x_pos,               // X coordinate 0~799
  output [9:0] o_y_pos,               // Y coordinate 0~599
  input        i_red,                 // RED value 0~1
  input        i_green,               // GEEN value 0~1
  input        i_blue,                // BLUE value 0~1
                                      // 2x2x2=8 color
  //
  output reg   o_vga_hs,              // Horizontal sync
  output reg   o_vga_vs,              // Vertical sync
  output       o_vga_r,               // RED 
  output       o_vga_g,               // GREEN 
  output       o_vga_b                // BLUE 
);

//++++++++++++++++++++++++++++++++++++++
// Parameter definition of 800x600@72Hz begins 
//++++++++++++++++++++++++++++++++++++++
// p = 56+120+64+800 = 1040 pixels/line
// l = 37+6+23+600 = 666 lines/screen
// s = 72 screens/second
// pixel_rate = p * l * s ~= 50M pixel/second
/* Horizontal parameter
 *       ________________________      ____
 * |Sync|Back|     Disp    |Front|Sync|Back|
 * |____|                        |____|
 *  _________ _____________ _______________
 *  Disp off |     Disp    |  Disp off     |
 *  _________ _____________________________
 *           |       Whole line            |
 *
 */
parameter H_FRONT    = 56;                        // Front porch
parameter H_SYNC     = 120;                       // Sync pulse
parameter H_BACK     = 64;                        // Back porch
parameter H_NOT_DISP = H_FRONT+H_SYNC+H_BACK;     // Disp off 
parameter H_DISP     = 800;                       // Disp  
parameter H_TOTAL    = H_NOT_DISP+H_DISP;         // Whole line

/* Vertical parameter
 *       ________________________      ____
 * |Sync|Back|     Disp    |Front|Sync|Back|
 * |____|                        |____|
 *  _________ _____________ _______________
 *  Disp off |     Disp    |  Disp off     |
 *  _________ _____________________________
 *           |       Whole Frame           |
 */
parameter V_FRONT    = 37;                        // Front porch
parameter V_SYNC     = 6;                         // Sync pulse
parameter V_BACK     = 23;                        // Back porch
parameter V_NOT_DISP = V_FRONT+V_SYNC+V_BACK;     // Disp off 
parameter V_DISP     = 600;                       // Disp  
parameter V_TOTAL    = V_NOT_DISP+V_DISP;         // Whole frame
//--------------------------------------
// Parameter definition of 800x600@72Hz ends
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
// generating horizontal and vertical signal begins 
//++++++++++++++++++++++++++++++++++++++
reg [10:0] h_cnt;

// the horizontal synchronous signal generator 
always @ (posedge i_clk, negedge i_rst_n)
  if (!i_rst_n)
  begin
    h_cnt    <= 0;
    o_vga_hs <= 1;
  end
  else
  begin
    // 0 ~ H_TOTAL-1
    if (h_cnt<H_TOTAL-1)
      h_cnt <= h_cnt + 1'b1;
    else
      h_cnt <= 0;
      
    if (h_cnt==0)
      o_vga_hs <= 0;
    if (h_cnt==H_SYNC-1)
      o_vga_hs <= 1;
  end

reg [10:0] v_cnt;

// the vertical synchronous signal generator 
always @ (posedge o_vga_hs, negedge i_rst_n)
  if (!i_rst_n)
  begin
    v_cnt    <= 0;
    o_vga_vs <= 1;
  end
  else
  begin
    // 0 ~ V_TOTAL-1
    if (v_cnt<V_TOTAL-1)
      v_cnt <= v_cnt+1'b1;
    else
      v_cnt <= 0;      
    
    if (v_cnt==0)
      o_vga_vs <= 0;
    if (v_cnt==V_SYNC-1)
      o_vga_vs <= 1;
  end
//--------------------------------------
// generating horizontal and vertical signal ends
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
// generating the pixel address begins 
//++++++++++++++++++++++++++++++++++++++
assign o_x_pos = (h_cnt>=H_SYNC+H_BACK) ? (h_cnt-H_SYNC-H_BACK) : 10'h0;
assign o_y_pos = (v_cnt>=V_SYNC+V_BACK) ? (v_cnt-V_SYNC-V_BACK) : 10'h0;
//--------------------------------------
// generating the pixel address ends 
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
// according to the pixel address, filling certain color begings 
//++++++++++++++++++++++++++++++++++++++
assign o_vga_r = (o_x_pos > 0) ? i_red   : 1'b0;
assign o_vga_g = (o_x_pos > 0) ? i_green : 1'b0;
assign o_vga_b = (o_x_pos > 0) ? i_blue  : 1'b0; 
//--------------------------------------
// according to the pixel address, filling certain color ends 
//--------------------------------------

endmodule
